'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var InitialAsyncState = {
  loading: true,
  result: undefined,
  error: undefined
};

var defaultSetLoading = function defaultSetLoading(asyncState) {
  return InitialAsyncState;
};
var defaultSetResult = function defaultSetResult(result, asyncState) {
  return {
    loading: false,
    result: result,
    error: undefined
  };
};
var defaultSetError = function defaultSetError(error, asyncState) {
  return {
    loading: false,
    result: undefined,
    error: error
  };
};

var DefaultOptions = {
  setLoading: defaultSetLoading,
  setResult: defaultSetResult,
  setError: defaultSetError
};
var normalizeOptions = function normalizeOptions(options) {
  return _extends({}, DefaultOptions, options);
};

var useAsyncState = function useAsyncState(options) {
  var _useState = react.useState(InitialAsyncState),
      _useState2 = slicedToArray(_useState, 2),
      value = _useState2[0],
      setValue = _useState2[1];

  return {
    value: value,
    setLoading: function setLoading() {
      return setValue(options.setLoading(value));
    },
    setResult: function setResult(result) {
      return setValue(options.setResult(result, value));
    },
    setError: function setError(error) {
      return setValue(options.setError(error, value));
    }
  };
};

var useIsMounted = function useIsMounted() {
  var mounted = true;
  react.useEffect(function () {
    return function () {
      return mounted = false;
    };
  }, []);
  return mounted;
};

var useCurrentPromise = function useCurrentPromise() {
  var ref = react.useRef(null);
  return {
    set: function set$$1(promise) {
      return ref.current = promise;
    },
    is: function is(promise) {
      return ref.current === promise;
    }
  };
};

var useAsync = function useAsync(asyncFunction) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var options = arguments[2];


  options = normalizeOptions(options);

  var AsyncState = useAsyncState(options);

  var isMounted = useIsMounted();

  var CurrentPromise = useCurrentPromise();

  // We only want to handle the promise result/error
  // if it is the last operation and the comp is still mounted
  var shouldHandlePromise = function shouldHandlePromise(p) {
    return isMounted && CurrentPromise.is(p);
  };

  var executeAsyncOperation = function executeAsyncOperation() {
    var promise = asyncFunction(params);
    CurrentPromise.set(promise);
    AsyncState.setLoading();
    promise.then(function (result) {
      if (shouldHandlePromise(promise)) {
        AsyncState.setResult(result);
      }
    }, function (error) {
      if (shouldHandlePromise(promise)) {
        AsyncState.setError(error);
      }
    });
  };

  react.useEffect(function () {
    executeAsyncOperation();
  }, params);

  var result = _extends({}, AsyncState.value, {
    execute: function execute() {
      return executeAsyncOperation();
    }
  });

  return result;
};

exports.useAsync = useAsync;
//# sourceMappingURL=index.js.map
